@@ -1,10 +1,10 @@
 #==============> Define S4 classes
 
 #' Enveomics: Recruitment Plot (2) - S4 Class
-#' 
+#'
 #' Enve-omics representation of Recruitment plots. This object can
 #' be produced by \code{\link{enve.recplot2}} and supports S4 method plot.
-#' 
+#'
 #' @slot counts \code{(matrix)} Counts as a two-dimensional histogram.
 #' @slot pos.counts.in \code{(numeric)} Counts of in-group hits per position bin.
 #' @slot pos.counts.out \code{(numeric)} Counts of out-group hits per position bin.
@@ -19,9 +19,9 @@
 #' @slot id.metric \code{(character}) Metric used as 'identity'.
 #' @slot id.ingroup \code{(logical}) Identity bins considered in-group.
 #' @slot call \code{(call)} Call producing this object.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @exportClass
 
 enve.RecPlot2 <- setClass("enve.RecPlot2",
@@ -44,10 +44,10 @@ enve.RecPlot2 <- setClass("enve.RecPlot2",
 );
 
 #' Enveomics: Recruitment Plot (2) Peak - S4 Class
-#' 
+#'
 #' Enve-omics representation of a peak in the sequencing depth histogram
 #' of a Recruitment plot (see \code{\link{enve.recplot2.findPeaks}}).
-#' 
+#'
 #' @slot dist \code{(character)}
 #' Distribution of the peak. Currently supported: \code{norm} (normal) and \code{sn}
 #' (skew-normal).
@@ -78,9 +78,9 @@ enve.RecPlot2 <- setClass("enve.RecPlot2",
 #' Best estimate available for the sequencing depth of the peak (centrality).
 #' @slot log \code{(logical)}
 #' Indicates if the estimation was performed in natural logarithm space.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @exportClass
 
 enve.RecPlot2.Peak <- setClass("enve.RecPlot2.Peak",
@@ -100,15 +100,15 @@ enve.RecPlot2.Peak <- setClass("enve.RecPlot2.Peak",
                                ));
 
 #' Attribute accessor
-#' 
-#' 
+#'
+#'
 #' @param x Object
 #' @param name Attribute name
 setMethod("$", "enve.RecPlot2", function(x, name) attr(x, name))
 
 #' Attribute accessor
-#' 
-#' 
+#'
+#'
 #' @param x Object
 #' @param name Attribute name
 setMethod("$", "enve.RecPlot2.Peak", function(x, name) attr(x, name))
@@ -116,9 +116,9 @@ setMethod("$", "enve.RecPlot2.Peak", function(x, name) attr(x, name))
 #==============> Define S4 methods
 
 #' Enveomics: Recruitment Plot (2)
-#' 
+#'
 #' Plots an \code{\link{enve.RecPlot2}} object.
-#' 
+#'
 #' @param x
 #' \code{\link{enve.RecPlot2}} object to plot.
 #' @param layout
@@ -192,18 +192,18 @@ setMethod("$", "enve.RecPlot2.Peak", function(x, name) attr(x, name))
 #' @param breaks.col
 #' Color of the vertical lines indicating sequence breaks.
 #' @param peaks.opts
-#' Options passed to \code{\link{enve.recplot2.findPeaks}}, 
+#' Options passed to \code{\link{enve.recplot2.findPeaks}},
 #' if \code{peaks.col} is not \code{NA}.
 #' @param ...
 #' Any other graphic parameters (currently ignored).
-#' 
-#' @return 
+#'
+#' @return
 #' Returns a list of \code{\link{enve.RecPlot2.Peak}} objects (see
-#' \code{\link{enve.recplot2.findPeaks}}). If \code{peaks.col=NA} or 
+#' \code{\link{enve.recplot2.findPeaks}}). If \code{peaks.col=NA} or
 #' \code{layout} doesn't include 4, returns \code{NA}.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @method plot enve.RecPlot2
 #' @export
 
@@ -241,37 +241,37 @@ plot.enve.RecPlot2 <- function
   pos.lim	<- pos.lim/pos.factor;
   lmat <- layout;
   for(i in 1:6) if(!any(layout==i)) lmat[layout>i] <- lmat[layout>i]-1;
-  
+
   layout(lmat, widths=widths, heights=heights);
   ori.mar <- par('mar');
-  
+
   # Essential vars
   counts	<- x$counts
-  
+
   id.ingroup	<- x$id.ingroup
   id.counts	<- x$id.counts
   id.breaks	<- x$id.breaks
   id.mids	<- (id.breaks[-length(id.breaks)]+id.breaks[-1])/2
   id.binsize	<- id.breaks[-1] - id.breaks[-length(id.breaks)]
-  
+
   pos.counts.in  <- x$pos.counts.in
   pos.counts.out <- x$pos.counts.out
   pos.breaks   <- x$pos.breaks/pos.factor
   pos.mids     <- (pos.breaks[-length(pos.breaks)]+pos.breaks[-1])/2
   pos.binsize  <- (pos.breaks[-1] - pos.breaks[-length(pos.breaks)])*pos.factor
-  
+
   seqdepth.in  <- pos.counts.in/pos.binsize
   seqdepth.out <- pos.counts.out/pos.binsize
   seqdepth.lim <- range(c(seqdepth.in[seqdepth.in>0],
                           seqdepth.out[seqdepth.out>0]))*c(1/2,2)
-  
+
   if(underlay.group){
     in.bg  <- do.call(rgb, c(as.list(col2rgb(in.col)),
                              list(maxColorValue=256, alpha=62)));
     out.bg <- do.call(rgb, c(as.list(col2rgb(out.col)[,1]),
                              list(maxColorValue=256, alpha=52)));
   }
-  
+
   # [1] Counts matrix
   if(any(layout==1)){
     par(mar=mar[['1']]);
@@ -290,7 +290,7 @@ plot.enve.RecPlot2 <- function
                                    length.out=1+length(palette)), add=TRUE);
     if(exists('1',panel.fun)) panel.fun[['1']]();
   }
-  
+
   # [2] Position histogram
   if(any(layout==2)){
     par(mar=mar[['2']]);
@@ -322,7 +322,7 @@ plot.enve.RecPlot2 <- function
                                     seqdepth.lim[1]*3/2, col=in.col,  border=NA);
     if(exists('2',panel.fun)) panel.fun[['2']]();
   }
-  
+
   # [3] Identity histogram
   if(any(layout==3)){
     par(mar=mar[['3']]);
@@ -358,7 +358,7 @@ plot.enve.RecPlot2 <- function
     }
     if(exists('3',panel.fun)) panel.fun[['3']]();
   }
-  
+
   # [4] Populations histogram
   peaks <- NA;
   if(any(layout==4)){
@@ -419,7 +419,7 @@ plot.enve.RecPlot2 <- function
     }
     if(exists('4',panel.fun)) panel.fun[['4']]();
   }
-  
+
   # [5] Color scale of the counts matrix (vertical)
   count.bins <- 10^seq(log10(min(counts[counts>0])), log10(max(counts)),
                        length.out=1+length(palette))
@@ -431,7 +431,7 @@ plot.enve.RecPlot2 <- function
          border=NA)
     if(exists('5',panel.fun)) panel.fun[['5']]();
   }
-  
+
   # [6] Color scale of the coutnts matrix (horizontal)
   if(any(layout==6)){
     par(mar=mar[['6']]);
@@ -441,7 +441,7 @@ plot.enve.RecPlot2 <- function
          border=NA);
     if(exists('6',panel.fun)) panel.fun[['6']]();
   }
-  
+
   par(mar=ori.mar);
   return(peaks);
 }
@@ -449,10 +449,10 @@ plot.enve.RecPlot2 <- function
 #==============> Define core functions
 
 #' Enveomics: Recruitment Plot (2)
-#' 
+#'
 #' Produces recruitment plots provided that \code{BlastTab.catsbj.pl} has
 #' been previously executed.
-#' 
+#'
 #' @param prefix
 #' Path to the prefix of the \code{BlastTab.catsbj.pl} output files. At
 #' least the files .rec and .lim must exist with this prefix.
@@ -496,99 +496,114 @@ plot.enve.RecPlot2 <- function
 #' Indicates if the function should report the advance.
 #' @param ...
 #' Any additional parameters supported by \code{\link{plot.enve.RecPlot2}}.
-#' 
+#'
 #' @return Returns an object of class \code{\link{enve.RecPlot2}}.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#' @author Kenji Gerhardt [aut]
+#'
 #' @export
 
 enve.recplot2 <- function(
   prefix,
-  plot=TRUE,
-  pos.breaks=1e3,
-  pos.breaks.tsv=NA,
-  id.breaks=300,
-  id.free.range=FALSE,
-  id.metric=c('identity', 'corrected identity', 'bit score'),
-  id.summary=sum,
-  id.cutoff=95,
-  threads=2,
-  verbose=TRUE,
+  plot = TRUE,
+  pos.breaks = 1e3,
+  pos.breaks.tsv = NA,
+  id.breaks = 60,
+  id.free.range = FALSE,
+  id.metric = c('identity', 'corrected identity', 'bit score'),
+  id.summary = sum,
+  id.cutoff = 95,
+  threads = 2,
+  verbose = TRUE,
   ...
 ){
   # Settings
   id.metric <- match.arg(id.metric);
-  
+
   #Read files
-  if(verbose) cat("Reading files.\n")
-  rec <- read.table(paste(prefix, ".rec", sep=""), sep="\t", comment.char="",
-                    quote="");
-  lim <- read.table(paste(prefix, ".lim", sep=""), sep="\t", comment.char="",
-                    quote="", as.is=TRUE);
-  
+  if (verbose) cat("Reading files.\n")
+  rec <- read.table(paste(prefix, ".rec", sep = ""),
+    sep = "\t", comment.char = "", quote = "");
+  lim <- read.table(paste(prefix, ".lim", sep = ""),
+    sep = "\t", comment.char = "", quote = "", as.is = TRUE);
+
   # Build matrix
-  if(verbose) cat("Building counts matrix.\n")
-  if(id.metric=="corrected identity" & ncol(rec)<6){
+  if (verbose) cat("Building counts matrix.\n")
+  if (id.metric == "corrected identity" & ncol(rec) < 6) {
     stop("Requesting corr. identity, but .rec file doesn't have 6th column")
   }
-  rec.idcol <- ifelse(id.metric=="identity", 3,
-                      ifelse(id.metric=="corrected identity", 6, 4))
+  rec.idcol <- ifelse(id.metric == "identity", 3,
+                      ifelse(id.metric == "corrected identity", 6, 4))
   pos.names <- as.character(NULL)
-  if(!is.na(pos.breaks.tsv)){
-    tmp <- read.table(pos.breaks.tsv, sep='\t', header=FALSE, as.is=TRUE)
-    pos.breaks <- as.numeric(tmp[,1])
-    if(ncol(tmp)>1) pos.names <- as.character(tmp[-1,2])
-  }else if(length(pos.breaks)==1){
-    if(pos.breaks>0){
-      pos.breaks <- seq(min(lim[,2]), max(lim[,3]), length.out=pos.breaks+1);
-    }else{
-      pos.breaks <- c(lim[1,2], lim[,3])
-      pos.names  <- lim[,1]
+  if (!is.na(pos.breaks.tsv)){
+    tmp <- read.table(pos.breaks.tsv, sep = "\t", header = FALSE, as.is = TRUE)
+    pos.breaks <- as.numeric(tmp[, 1])
+    if (ncol(tmp) > 1) pos.names <- as.character(tmp[-1, 2])
+  } else if (length(pos.breaks) == 1) {
+    if (pos.breaks > 0){
+      pos.breaks <- seq(min(lim[, 2]), max(lim[, 3]), length.out = pos.breaks + 1)
+    } else {
+      pos.breaks <- c(lim[1, 2], lim[, 3])
+      pos.names  <- lim[, 1]
     }
   }
-  if(length(id.breaks)==1){
-    id.range.v <- rec[,rec.idcol]
-    if(!id.free.range) id.range.v <- c(id.range.v,70,100)
+  if (length(id.breaks) == 1) {
+    id.range.v <- rec[, rec.idcol]
+    if (!id.free.range) id.range.v <- c(id.range.v, 70, 100)
     id.range.v <- range(id.range.v)
-    id.breaks <- seq(id.range.v[1], id.range.v[2], length.out=id.breaks+1);
+    id.breaks <- seq(id.range.v[1], id.range.v[2], length.out = id.breaks + 1)
   }
-  
+
   # Run in parallel
-  if(nrow(rec) < 200) threads <- 1 # It doesn't worth the overhead
-  cl    <- makeCluster(threads)
-  rec.l <- list()
-  thl   <- ceiling(nrow(rec)/threads)
-  for(i in 0:(threads-1)){
-    rec.l[[i+1]] <- list(
-      rec=rec[ (i*thl+1):min(((i+1)*thl),nrow(rec)), ],
-      verbose=ifelse(i==0, verbose, FALSE))
+  # If they already set threads to 1 manually, there's no point in launching
+  # clusters, it's just slower. Ditto for small files.
+  if (nrow(rec) < 75000 | threads == 1) {
+    # Coerces rec into a form that __counts is happy about
+    rec.l <- list()
+    rec.l[[1]] <- list(rec = rec, verbose = FALSE)
+
+    # No need to make a temporary variable, there's only one return for sure
+    # and it's not a list because it isn't coming back from an apply
+    counts <- enve.recplot2.__counts(
+      rec.l[[1]], pos.breaks = pos.breaks, id.breaks = id.breaks,
+      rec.idcol = rec.idcol)
+  } else {
+    cl <- makeCluster(threads)
+    rec.l <- list()
+    thl <- ceiling(nrow(rec)/threads)
+    for (i in 0:(threads - 1)) {
+      rec.l[[i + 1]] <- list(
+        rec = rec[(i * thl + 1):min(((i + 1) * thl), nrow(rec)), ],
+        verbose = ifelse(i == 0, verbose, FALSE))
+    }
+    counts.l <- clusterApply(
+      cl, rec.l, enve.recplot2.__counts, pos.breaks = pos.breaks,
+      id.breaks = id.breaks, rec.idcol = rec.idcol)
+    stopCluster(cl) # No spooky ghost clusters
+
+    counts <- counts.l[[1]]
+    for (i in 2:threads) counts <- counts + counts.l[[i]]
   }
-  counts.l <- clusterApply(cl, rec.l, enve.recplot2.__counts,
-                           pos.breaks=pos.breaks, id.breaks=id.breaks,
-                           rec.idcol=rec.idcol)
-  counts   <- counts.l[[1]]
-  if(threads>1) for(i in 2:threads) counts <- counts + counts.l[[i]]
-  stopCluster(cl)
-  
+
   # Estimate 1D histograms
-  if(verbose) cat("Building histograms.\n")
-  id.mids	<- (id.breaks[-length(id.breaks)]+id.breaks[-1])/2;
+  if (verbose) cat("Building histograms.\n")
+  id.mids	<- (id.breaks[-length(id.breaks)] + id.breaks[-1])/2;
   id.ingroup	<- (id.mids > id.cutoff);
   id.counts	<- apply(counts, 2, id.summary);
-  pos.counts.in   <- apply(counts[,id.ingroup], 1, sum);
-  pos.counts.out  <- apply(counts[,!id.ingroup], 1, sum);
-  
+  pos.counts.in   <- apply(counts[, id.ingroup], 1, sum);
+  pos.counts.out  <- apply(counts[, !id.ingroup], 1, sum);
+
   # Plot and return
   recplot <- new('enve.RecPlot2',
-                 counts=counts, id.counts=id.counts, pos.counts.in=pos.counts.in,
-                 pos.counts.out=pos.counts.out,
-                 id.breaks=id.breaks, pos.breaks=pos.breaks, pos.names=pos.names,
-                 seq.breaks=c(lim[1,2], lim[,3]), seq.names=lim[,1],
-                 id.ingroup=id.ingroup,id.metric=id.metric,
-                 call=match.call());
-  if(plot){
-    if(verbose) cat("Plotting.\n")
+                 counts = counts, id.counts = id.counts,
+                 pos.counts.in = pos.counts.in, pos.counts.out = pos.counts.out,
+                 id.breaks = id.breaks, pos.breaks = pos.breaks,
+                 pos.names = pos.names, seq.breaks = c(lim[1, 2], lim[, 3]),
+                 seq.names = lim[, 1], id.ingroup = id.ingroup,
+                 id.metric = id.metric, call = match.call());
+  if (plot) {
+    if (verbose) cat("Plotting.\n")
     peaks <- plot(recplot, ...);
     attr(recplot, "peaks") <- peaks
   }
@@ -596,10 +611,10 @@ enve.recplot2 <- function(
 }
 
 #' Enveomics: Recruitment Plot (2) Peak Finder
-#' 
+#'
 #' Identifies peaks in the population histogram potentially indicating
 #' sub-population mixtures.
-#' 
+#'
 #' @param x
 #' An \code{\link{enve.RecPlot2}} object.
 #' @param method
@@ -607,7 +622,7 @@ enve.recplot2 <- function(
 #' \itemize{
 #'    \item \strong{emauto}
 #'    (Expectation-Maximization with auto-selection of components)
-#'    \item \strong{em} 
+#'    \item \strong{em}
 #'    (Expectation-Maximization)
 #'    \item \strong{mower}
 #'    (Custom distribution-mowing method)
@@ -615,11 +630,11 @@ enve.recplot2 <- function(
 #' @param ...
 #' Any additional parameters supported by
 #' \code{\link{enve.recplot2.findPeaks}}.
-#' 
+#'
 #' @return Returns a list of \code{\link{enve.RecPlot2.Peak}} objects.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' export
 
 enve.recplot2.findPeaks <- function(
@@ -640,11 +655,11 @@ enve.recplot2.findPeaks <- function(
 }
 
 #' Enveomics: Recruitment Plot (2) Emauto Peak Finder
-#' 
+#'
 #' Identifies peaks in the population histogram using a Gaussian Mixture
 #' Model Expectation Maximization (GMM-EM) method with number of components
 #' automatically detected.
-#' 
+#'
 #' @param x
 #' An \code{\link{enve.RecPlot2}} object.
 #' @param components
@@ -660,11 +675,11 @@ enve.recplot2.findPeaks <- function(
 #' Display (mostly debugging) information.
 #' @param ...
 #' Any additional parameters supported by \code{\link{enve.recplot2.findPeaks.em}}.
-#' 
+#'
 #' @return Returns a list of \code{\link{enve.RecPlot2.Peak}} objects.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.emauto <- function(
@@ -689,7 +704,7 @@ enve.recplot2.findPeaks.emauto <- function(
                                                  verbose, ...)
   }
   if(length(best[['peaks']])==0) return(list())
-  
+
   seqdepths.r <- signif(log(sapply(best[['peaks']],
                                    function(x) x$seq.depth)), merge.tol)
   distinct <- length(unique(seqdepths.r))
@@ -707,10 +722,10 @@ enve.recplot2.findPeaks.emauto <- function(
 }
 
 #' Enveomics: Recruitment Plot (2) Em Peak Finder
-#' 
+#'
 #' Identifies peaks in the population histogram using a Gaussian Mixture
 #' Model Expectation Maximization (GMM-EM) method.
-#' 
+#'
 #' @param x
 #' An \code{\link{enve.RecPlot2}} object.
 #' @param max.iter
@@ -722,23 +737,23 @@ enve.recplot2.findPeaks.emauto <- function(
 #' @param rm.top
 #' Top-values to remove before finding peaks, as a quantile probability.
 #' This step is useful to remove highly conserved regions, but can be
-#' turned off by setting \code{rm.top=0}. The quantile is determined 
+#' turned off by setting \code{rm.top=0}. The quantile is determined
 #' \strong{after} removing zero-coverage windows.
 #' @param verbose
 #' Display (mostly debugging) information.
 #' @param init
 #' Initialization parameters. By default, these are derived from k-means
-#' clustering. A named list with vectors for \code{mu}, \code{sd}, and 
+#' clustering. A named list with vectors for \code{mu}, \code{sd}, and
 #' \code{alpha}, each of length \code{components}.
 #' @param log
 #' Logical value indicating if the estimations should be performed in
 #' natural logarithm units. Do not change unless you know what you're
 #' doing.
-#' 
+#'
 #' @return Returns a list of \code{\link{enve.RecPlot2.Peak}} objects.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.em <- function(
@@ -751,13 +766,13 @@ enve.recplot2.findPeaks.em <- function(
   init,
   log=TRUE
 ){
-  
+
   # Essential vars
   pos.binsize  <- x$pos.breaks[-1] - x$pos.breaks[-length(x$pos.breaks)]
   lsd1  <- (x$pos.counts.in/pos.binsize)[ x$pos.counts.in > 0 ]
   lsd1 <- lsd1[ lsd1 < quantile(lsd1, 1-rm.top, names=FALSE) ]
   if(log) lsd1 <- log(lsd1)
-  
+
   # 1. Initialize
   if(missing(init)){
     km.clust <- kmeans(lsd1, components)$cluster
@@ -770,7 +785,7 @@ enve.recplot2.findPeaks.em <- function(
   m.step <- init
   ll <- c()
   cur.ll <- -Inf
-  
+
   for(i in 1:max.iter){
     # 2/3. EM
     e.step <- enve.recplot2.findPeaks.__em_e(lsd1, m.step)
@@ -782,7 +797,7 @@ enve.recplot2.findPeaks.em <- function(
     if(verbose) cat(i, '\t| LL =', cur.ll, '\t| LL.diff =', ll.diff, '\n')
     if(ll.diff <= ll.diff.res) break
   }
-  
+
   # Return
   peaks <- list()
   for(i in 1:components){
@@ -798,10 +813,10 @@ enve.recplot2.findPeaks.em <- function(
 }
 
 #' Enveomics: Recruitment Plot (2) Mowing Peak Finder
-#' 
+#'
 #' Identifies peaks in the population histogram potentially indicating
 #' sub-population mixtures, using a custom distribution-mowing method.
-#' 
+#'
 #' @param x
 #' An \code{\link{enve.RecPlot2}} object.
 #' @param min.points
@@ -814,16 +829,16 @@ enve.recplot2.findPeaks.em <- function(
 #' Ignored. For backwards compatibility.
 #' @param fitdist.opts.sn
 #' Options passed to \code{fitdist} to estimate the standard deviation if
-#' \code{with.skewness=TRUE}. Note that the \code{start} parameter will be 
+#' \code{with.skewness=TRUE}. Note that the \code{start} parameter will be
 #' ammended with \code{xi=estimated} mode for each peak.
 #' @param fitdist.opts.norm
 #' Options passed to \code{fitdist} to estimate the standard deviation if
-#' \code{with.skewness=FALSE}. Note that the \code{start} parameter will be 
+#' \code{with.skewness=FALSE}. Note that the \code{start} parameter will be
 #' ammended with \code{mean=estimated} mode for each peak.
 #' @param rm.top
 #' Top-values to remove before finding peaks, as a quantile probability.
 #' This step is useful to remove highly conserved regions, but can be
-#' turned off by setting \code{rm.top=0}. The quantile is determined 
+#' turned off by setting \code{rm.top=0}. The quantile is determined
 #' \strong{after} removing zero-coverage windows.
 #' @param with.skewness
 #' Allow skewness correction of the peaks. Typically, the
@@ -840,7 +855,7 @@ enve.recplot2.findPeaks.em <- function(
 #' differences between parameters in one round and those from two rounds
 #' earlier (to avoid infinite loops from approximation).
 #' @param merge.logdist
-#' Maximum value of \code{|log-ratio|} between centrality parameters in peaks 
+#' Maximum value of \code{|log-ratio|} between centrality parameters in peaks
 #' to attempt merging. The default of ~0.22 corresponds to a maximum
 #' difference of 25\%.
 #' @param verbose
@@ -849,11 +864,11 @@ enve.recplot2.findPeaks.em <- function(
 #' Logical value indicating if the estimations should be performed in
 #' natural logarithm units. Do not change unless you know what you're
 #' doing.
-#' 
+#'
 #' @return Returns a list of \code{\link{enve.RecPlot2.Peak}} objects.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.mower <- function(
@@ -873,7 +888,7 @@ enve.recplot2.findPeaks.mower <- function(
   verbose=FALSE,
   log=TRUE
 ){
-  
+
   # Essential vars
   pos.binsize	<- x$pos.breaks[-1] - x$pos.breaks[-length(x$pos.breaks)];
   seqdepth.in	<- x$pos.counts.in/pos.binsize;
@@ -889,11 +904,11 @@ enve.recplot2.findPeaks.mower <- function(
                      mlv.opts=mlv.opts, fitdist.opts=fitdist.opts, with.skewness=with.skewness,
                      optim.rounds=optim.rounds, optim.epsilon=optim.epsilon, verbose=verbose,
                      n.total=length(lsd1), merge.logdist=merge.logdist, log=log)
-  
+
   # Find seed peaks
   if(verbose) cat('Mowing peaks for n =',length(lsd1),'\n')
   peaks <- enve.recplot2.findPeaks.__mower(peaks.opts);
-  
+
   # Merge overlapping peaks
   if(verbose) cat('Trying to merge',length(peaks),'peaks\n')
   merged <- (length(peaks)>1)
@@ -925,7 +940,7 @@ enve.recplot2.findPeaks.mower <- function(
     peaks <- peaks2
     if(length(peaks)==1) break
   }
-  
+
   if(verbose) cat('Found',length(peaks),'peak(s)\n')
   return(peaks);
 }
@@ -933,14 +948,14 @@ enve.recplot2.findPeaks.mower <- function(
 #==============> Define utils
 
 #' Enveomics: Recruitment Plot (2) Core Peak Finder
-#' 
+#'
 #' Finds the peak in a list of peaks that is most likely to represent the
 #' "core genome" of a population.
-#' 
+#'
 #' @param x \code{list} of \code{\link{enve.RecPlot2.Peak}} objects.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.corePeak <- function
@@ -970,16 +985,16 @@ enve.recplot2.corePeak <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Change Cutoff
-#' 
+#'
 #' Change the intra-species cutoff of an existing recruitment plot.
-#' 
+#'
 #' @param rp
 #' \code{\link{enve.RecPlot2}} object.
 #' @param new.cutoff
 #' New cutoff to use.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.changeCutoff <- function
@@ -1000,10 +1015,10 @@ enve.recplot2.changeCutoff <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Window Depth Threshold
-#' 
+#'
 #' Identifies the threshold below which windows should be identified as
 #' variable or absent.
-#' 
+#'
 #' @param rp
 #' Recruitment plot, an \code{\link{enve.RecPlot2}} object.
 #' @param peak
@@ -1015,13 +1030,13 @@ enve.recplot2.changeCutoff <- function
 #' sequencing depth.
 #' @param significance
 #' Significance threshold (alpha) to select windows.
-#' 
-#' @return 
+#'
+#' @return
 #' Returns a float. The units are depth if the peaks were estimated in
 #' linear scale, or log-depth otherwise (\code{peak$log}).
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.windowDepthThreshold <- function
@@ -1035,15 +1050,15 @@ enve.recplot2.windowDepthThreshold <- function
   par[["p"]] <- ifelse(lower.tail, significance, 1-significance)
   thr <- do.call(ifelse(length(par)==4, qsn, qnorm), par)
   if(peak$log) thr <- exp(thr)
-  
+
   return(thr)
 }
 
 #' Enveomics: Recruitment Plot (2) Extract Windows
-#' 
+#'
 #' Extract windows significantly below (or above) the peak in sequencing
 #' depth.
-#' 
+#'
 #' @param rp
 #' Recruitment plot, a \code{\link{enve.RecPlot2}} object.
 #' @param peak
@@ -1058,19 +1073,19 @@ enve.recplot2.windowDepthThreshold <- function
 #' @param seq.names
 #' Returns subject sequence names instead of a vector of Booleans. If
 #' the recruitment plot was generated with named position bins (e.g, using
-#' \code{pos.breaks=0} or a two-column \code{pos.breaks.tsv}), it returns a 
-#' vector of characters (the sequence identifiers), otherwise it returns a 
+#' \code{pos.breaks=0} or a two-column \code{pos.breaks.tsv}), it returns a
+#' vector of characters (the sequence identifiers), otherwise it returns a
 #' data.frame with a name column and two columns of coordinates.
-#' 
-#' @return 
-#' Returns a vector of logicals if \code{seq.names=FALSE}. If 
-#' \code{seq.names=TRUE}, it returns a vector of characters if the object has 
+#'
+#' @return
+#' Returns a vector of logicals if \code{seq.names=FALSE}. If
+#' \code{seq.names=TRUE}, it returns a vector of characters if the object has
 #' \code{pos.names} defined, or a data.frame with four columns otherwise:
 #' \code{name.from}, \code{name.to}, \code{pos.from}, and \code{pos.to}
 #' (see \code{\link{enve.recplot2.coordinates}}).
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.extractWindows <- function
@@ -1082,7 +1097,7 @@ enve.recplot2.extractWindows <- function
 ){
   # Determine the threshold
   thr <- enve.recplot2.windowDepthThreshold(rp, peak, lower.tail, significance)
-  
+
   # Select windows past the threshold
   seqdepth.in <- enve.recplot2.seqdepth(rp)
   if(lower.tail){
@@ -1090,7 +1105,7 @@ enve.recplot2.extractWindows <- function
   }else{
     sel <- seqdepth.in > thr
   }
-  
+
   # seq.names=FALSE
   if(!seq.names) return(sel)
   # seq.names=TRUE and pos.names defined
@@ -1100,10 +1115,10 @@ enve.recplot2.extractWindows <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Compare Identities
-#' 
-#' Compare the distribution of identities between two \code{\link{enve.RecPlot2}} 
+#'
+#' Compare the distribution of identities between two \code{\link{enve.RecPlot2}}
 #' objects.
-#' 
+#'
 #' @param x
 #' First \code{\link{enve.RecPlot2}} object.
 #' @param y
@@ -1128,9 +1143,9 @@ enve.recplot2.extractWindows <- function
 #' @param max.deviation
 #' Maximum mean deviation between identity breaks tolerated (as percent
 #' identity). Difference in number of \code{id.breaks} is never tolerated.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.compareIdentities <- function
@@ -1183,25 +1198,25 @@ enve.recplot2.compareIdentities <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Coordinates
-#' 
+#'
 #' Returns the sequence name and coordinates of the requested position bins.
-#' 
+#'
 #' @param x
 #' \code{\link{enve.RecPlot2}} object.
 #' @param bins
 #' Vector of selected bins to return. It can be a vector of logical values
 #' with the same length as \code{x$pos.breaks-1} or a vector of integers. If
 #' missing, returns the coordinates of all windows.
-#' 
-#' @return 
+#'
+#' @return
 #' Returns a data.frame with four columns: \code{name.from (character), pos.from
 #' (numeric) name.to (character), and pos.to (numeric)}. The first two
 #' correspond to sequence and position of the start point of the bin, the
 #' last two correspond to the sequence and position of the end point of the
 #' bin.
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.coordinates <- function
@@ -1213,9 +1228,9 @@ enve.recplot2.coordinates <- function
   if(missing(bins)) bins <- rep(TRUE, length(x$pos.breaks)-1)
   if(!is.vector(bins)) stop("'bins' must be a vector")
   if(inherits(bins, "logical")) bins <- which(bins)
-  
+
   y <- data.frame(stringsAsFactors=FALSE, row.names=bins)
-  
+
   for(i in 1:length(bins)){
     j <- bins[i]
     # Concatenated coordinates
@@ -1235,14 +1250,14 @@ enve.recplot2.coordinates <- function
       y[i, 'pos.to']   <- ceiling(x$seq.breaks[sb.to] + cc[2] - 1)
     }
   }
-  
+
   return(y)
 }
 
 #' Enveomics: Recruitment Plot (2) Sequencing Depth
-#' 
+#'
 #' Calculate the sequencing depth of the given window(s).
-#' 
+#'
 #' @param x
 #' \code{\link{enve.RecPlot2}} object.
 #' @param sel
@@ -1252,12 +1267,12 @@ enve.recplot2.coordinates <- function
 #' A logical indicating if the sequencing depth is to be estimated only
 #' with low-identity matches. By default, only high-identity matches are
 #' used.
-#' 
-#' @return 
-#' Returns a numeric vector of sequencing depths (in bp/bp). 
-#' 
+#'
+#' @return
+#' Returns a numeric vector of sequencing depths (in bp/bp).
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.seqdepth <- function
@@ -1281,10 +1296,10 @@ enve.recplot2.seqdepth <- function
 }
 
 #' Enveomics: Recruitment Plot (2) ANI Estimate
-#' 
+#'
 #' Estimate the Average Nucleotide Identity from reads (ANIr) from a
 #' recruitment plot.
-#' 
+#'
 #' @param x
 #' \code{\link{enve.RecPlot2}} object.
 #' @param range
@@ -1293,9 +1308,9 @@ enve.recplot2.seqdepth <- function
 #' recruitment plots can also be built with bit-scores). To use only
 #' intra-population matches (with identities), use c(95,100). To use only
 #' inter-population values, use c(0,95).
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.ANIr <- function
@@ -1315,50 +1330,70 @@ enve.recplot2.ANIr <- function
 #==============> Define internal functions
 
 #' Enveomics: Recruitment Plot (2) Internal Ancillary Function
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.recplot2}}).
-#' 
+#'
 #' @param x \code{\link{enve.RecPlot2}} object
 #' @param pos.breaks Position breaks
 #' @param id.breaks Identity breaks
 #' @param rec.idcol Identity column to use
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#' @author Kenji Gerhardt [aut]
+#'
 #' @export
 
 enve.recplot2.__counts <- function
-(x, pos.breaks, id.breaks, rec.idcol){
-  rec <- x$rec
+(x, pos.breaks, id.breaks, rec.idcol) {
+  rec2 <- x$rec
   verbose <- x$verbose
-  counts <- matrix(0, nrow=length(pos.breaks)-1, ncol=length(id.breaks)-1);
-  for(i in 1:nrow(rec)){
-    if(verbose & i%%100==0) cat("   [",signif(i*100/nrow(rec),3),"% ]   \r");
-    y.bin <- which(
-      rec[i,rec.idcol]>=id.breaks[-length(id.breaks)] &
-        rec[i,rec.idcol]<=id.breaks[-1])[1] ;
-    for(pos in rec[i,1]:rec[i,2]){
-      x.bin <- which(
-        pos>=pos.breaks[-length(pos.breaks)] & pos<=pos.breaks[-1])[1] ;
-      counts[x.bin, y.bin] <- counts[x.bin, y.bin]+1 ;
-    }
+
+  # get counts of how many occurrences of each genome pos.bin there are per read
+  x.bins <- mapply(
+    function(start, end) {
+      list(rle(findInterval(start:end, pos.breaks, left.open = T)))
+    }, rec2[, 1], rec2[, 2])
+
+  # find the single y bin for each row, replicates it at the correct places to
+  # the number of distinct bins found in its row
+  y.bins <- rep(findInterval(rec2[, rec.idcol], id.breaks, left.open = T),
+                times = unlist(lapply(x.bins, function(a) length(a$lengths))))
+
+  # x.bins_counts is the number of occurrences of each bin a row contains,
+  # per row, then unlisted
+  x.bins_counts <- unlist(lapply(x.bins, function(a) a$lengths))
+
+  # these are the pos. in. genome bins that each count in x.bins_counts falls into
+  x.bins <- unlist(lapply(x.bins, function(a) a$values))
+
+  # much more efficient counts implementation in R using lists instead of a matrix:
+  counts <- lapply(
+    1:(length(pos.breaks) - 1),
+    function(col_len) rep(0, length(id.breaks) - 1))
+
+  # accesses the correct list in counts by x.bin, then
+  # accesses the position in that row by y.bins and adds the new count
+  for(i in 1:length(x.bins)){
+    counts[[x.bins[i]]][y.bins[i]] <- counts[[x.bins[i]]][y.bins[i]] + x.bins_counts[i]
   }
-  return(counts);
+
+  counts <- do.call(rbind, counts)
+  return(counts)
 }
 
 #' Enveomics: Recruitment Plot (2) EMauto Peak Finder - Internal Ancillary Function
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.recplot2.findPeaks.emauto}}).
-#' 
+#'
 #' @param x \code{\link{enve.RecPlot2}} object
 #' @param comp Components
 #' @param do_crit Function estimating the criterion
 #' @param best Best solution thus far
 #' @param verbose If verbose
 #' @param ... Additional parameters for \code{\link{enve.recplot2.findPeaks.em}}
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.__emauto_one <- function
@@ -1378,14 +1413,14 @@ enve.recplot2.findPeaks.__emauto_one <- function
 }
 
 #' Enveomics: Recruitment Plot (2) EM Peak Finder - Internal Ancillary Function Expectation
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.recplot2.findPeaks.em}}).
-#' 
+#'
 #' @param x Vector of log-transformed sequencing depths
 #' @param theta Parameters list
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.__em_e <- function
@@ -1397,19 +1432,19 @@ enve.recplot2.findPeaks.__em_e <- function
                                               theta[['sd']][i])*theta[['alpha']][i]))
   sum.of.components <- rowSums(product)
   posterior <- product / sum.of.components
-  
+
   return(list(ll=sum(log(sum.of.components)), posterior=posterior))
 }
 
 #' Enveomics: Recruitment Plot (2) Em Peak Finder - Internal Ancillary Function Maximization
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.recplot2.findPeaks.em}}).
-#' 
+#'
 #' @param x Vector of log-transformed sequencing depths
 #' @param posterior Posterior probability
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.__em_m <- function
@@ -1424,15 +1459,15 @@ enve.recplot2.findPeaks.__em_m <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Peak S4 Class - Internal Ancillary Function
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.RecPlot2.Peak}}).
-#' 
+#'
 #' @param x \code{\link{enve.RecPlot2.Peak}} object
 #' @param mids Midpoints
 #' @param counts Counts
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.__peakHist <- function
@@ -1451,9 +1486,9 @@ enve.recplot2.__peakHist <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Mowing Peak Finder - Internal Ancillary Function 1
-#' 
+#'
 #' Internall ancillary function (see \code{\link{enve.recplot2.findPeaks.mower}}).
-#' 
+#'
 #' @param lsd1 Vector of log-transformed sequencing depths
 #' @param min.points Minimum number of points
 #' @param quant.est Quantile estimate
@@ -1466,9 +1501,9 @@ enve.recplot2.__peakHist <- function
 #' @param merge.logdist Attempted \code{merge.logdist} parameter
 #' @param verbose If verbose
 #' @param log If log-transformed depths
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.__mow_one <- function
@@ -1476,7 +1511,7 @@ enve.recplot2.findPeaks.__mow_one <- function
  optim.rounds, optim.epsilon, n.total, merge.logdist, verbose, log
 ){
   dist	<- ifelse(with.skewness, 'sn', 'norm');
-  
+
   # Find peak
   o <- mlv.opts; o$x = lsd1;
   mode1 <- median(lsd1); # mode1 <- do.call(mlv, o)$M;
@@ -1484,7 +1519,7 @@ enve.recplot2.findPeaks.__mow_one <- function
   param.hat <- fitdist.opts$start; last.hat <- param.hat;
   lim <- NA;
   if(with.skewness){ param.hat$xi <- mode1 }else{ param.hat$mean <- mode1 }
-  
+
   # Refine peak parameters
   for(round in 1:optim.rounds){
     param.hat[[ 1 ]] <- param.hat[[ 1 ]]/diff(quant.est)# <- expand dispersion
@@ -1513,7 +1548,7 @@ enve.recplot2.findPeaks.__mow_one <- function
   }
   if(verbose) cat('\n')
   if(is.na(param.hat[1]) | is.na(lim[1])) return(NULL);
-  
+
   # Mow distribution
   lsd2 <- c();
   lsd.pop <- c();
@@ -1534,7 +1569,7 @@ enve.recplot2.findPeaks.__mow_one <- function
     lsd.pop <- c(lsd.pop, values[in.peak])
   }
   if(length(lsd.pop) < min.points) return(NULL)
-  
+
   # Return peak
   attr(peak, 'values') <- lsd.pop
   attr(peak, 'values.res') <- lsd2
@@ -1549,13 +1584,13 @@ enve.recplot2.findPeaks.__mow_one <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Mowing Peak Finder - Internal Ancillary Function 2
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.recplot2.findPeaks.mower}}).
-#' 
+#'
 #' @param peaks.opts List of options for \code{\link{enve.recplot2.findPeaks.__mow_one}}
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.findPeaks.__mower <- function
@@ -1571,14 +1606,14 @@ enve.recplot2.findPeaks.__mower <- function
 }
 
 #' Enveomics: Recruitment Plot (2) Peak Finder - Internal Ancillary Function
-#' 
+#'
 #' Internal ancillary function (see \code{\link{enve.recplot2.findPeaks}}).
-#' 
+#'
 #' @param peak Query \code{\link{enve.RecPlot2.Peak}} object
 #' @param peaks list of \code{\link{enve.RecPlot2.Peak}} objects
-#' 
+#'
 #' @author Luis M. Rodriguez-R [aut, cre]
-#' 
+#'
 #' @export
 
 enve.recplot2.__whichClosestPeak <- function
@@ -1590,4 +1625,3 @@ enve.recplot2.__whichClosestPeak <- function
   dist[ dist==0 ] <- Inf
   return(which.min(dist))
 }
-
